package adminkindboxreqvalidator

import (
	"context"
	"errors"
	"fmt"
	"time"

	"git.gocasts.ir/ebhomengo/niki/entity"
	addressparam "git.gocasts.ir/ebhomengo/niki/param/admin/address"
	adminbenefactoreparam "git.gocasts.ir/ebhomengo/niki/param/admin/benefactor"
	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box_req"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateAddRequest(ctx context.Context, req param.KindBoxReqAddRequest) (map[string]string, error) {
	const op = "adminkindboxreqvalidator.ValidateAddRequest"

	if err := validation.ValidateStruct(&req,

		validation.Field(&req.CountRequested, validation.Required, validation.Min(uint(MinKindBoxReq)), validation.Max(uint(MaxKindBoxReq))),

		validation.Field(&req.BenefactorID,
			validation.Required,
			validation.By(v.doesBenefactorExist(ctx))),

		validation.Field(&req.KindBoxType,
			validation.Required,
			validation.By(v.doesTypeExist)),

		validation.Field(&req.DeliverAddressID,
			validation.Required,
			validation.By(v.doesAddressExist(ctx, req.BenefactorID))),

		validation.Field(&req.DeliverReferDate,
			validation.Required,
			validation.By(v.isDateValid),
		),

		validation.Field(&req.DeliverReferTimeID,
			validation.Required,
			validation.By(v.isReferTimeIDValid(ctx)),
		),
	); err != nil {

		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)

	}

	return map[string]string{}, nil
}

func (v Validator) doesBenefactorExist(ctx context.Context) validation.RuleFunc {
	return func(value interface{}) error {
		benefactorID, ok := value.(uint)
		if !ok {
			return fmt.Errorf(errmsg.ErrorMsgSomethingWentWrong)
		}
		_, err := v.benefactorSvc.BenefactorExistByID(ctx, adminbenefactoreparam.BenefactorExistByIDRequest{ID: benefactorID})
		if err != nil {
			return fmt.Errorf(errmsg.ErrorMsgNotFound)
		}

		return nil
	}
}

func (v Validator) doesTypeExist(value interface{}) error {
	KindBoxType, ok := value.(entity.KindBoxType)
	if !ok {
		return fmt.Errorf(errmsg.ErrorMsgSomethingWentWrong)
	}
	if !KindBoxType.IsValid() {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}

func (v Validator) doesAddressExist(ctx context.Context, benefactorID uint) validation.RuleFunc {
	return func(value interface{}) error {
		addressID, ok := value.(uint)
		if !ok {
			return fmt.Errorf(errmsg.ErrorMsgSomethingWentWrong)
		}
		address, err := v.addressSvc.GetAddressByID(ctx, addressparam.AddressGetRequest{AddressID: addressID})
		if err != nil {
			return fmt.Errorf(errmsg.ErrorMsgSomethingWentWrong)
		}
		if address.Address.BenefactorID != benefactorID {
			return fmt.Errorf(errmsg.ErrorMsgNotFound)
		}

		return nil
	}
}

func (v Validator) isDateValid(value interface{}) error {
	date, ok := value.(time.Time)
	if !ok {
		return fmt.Errorf(errmsg.ErrorMsgSomethingWentWrong)
	}

	if date.Before(time.Now()) {
		return fmt.Errorf(errmsg.ErrorMsgInvalidInput)
	}

	return nil
}
